// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/cards/widgets/card_text.dart';

const _titlesLineSpacing = 5.0;
const _bodyTopSpacing = 25.0;

class PricingCardTitles extends StatelessWidget {
  const PricingCardTitles({
    this.axis,
    this.title,
    this.pricing,
    this.titleStyle,
    this.pricingStyle,
    this.periodStyle,
    super.key,
  });

  /// The axis of the card header.
  final Axis? axis;

  /// Title of the card
  ///
  /// If [axis] is [Axis.vertical] then icon of the title will be displayed
  /// above the title.
  final PricingLine? title;

  /// Pricing of the card
  final Pricing? pricing;

  /// Styles the title of the card.
  final TextStyle? titleStyle;

  /// Styles the pricing of the card.
  final TextStyle? pricingStyle;

  /// Styles the period of the card.
  final TextStyle? periodStyle;

  @override
  Widget build(BuildContext context) {
    final price = (pricing != null)
        ? CardText.fromWrapper(
            pricing!.price,
            style: pricingStyle,
            textType: TextType.title,
          )
        : null;

    final period = ((pricing != null) && (pricing?.period != null))
        ? CardText.fromWrapper(
            pricing!.period!,
            style: periodStyle,
            textType: TextType.subtitle,
          )
        : null;

    return Column(
      children: [
        if (title != null) ...[
          if (axis == Axis.vertical) ...[
            if (title?.icon != null) ...[
              title!.icon!,
              const Gap(_titlesLineSpacing),
            ],
            CardText.fromWrapper(
              title!.data,
              style: titleStyle,
              textType: TextType.title,
            ),
            const Gap(_bodyTopSpacing),
          ] else ...[
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                if (title?.icon != null) ...[
                  title!.icon!,
                  const Gap(_titlesLineSpacing),
                ],
                CardText.fromWrapper(
                  title!.data,
                  style: titleStyle,
                  textType: TextType.title,
                ),
              ],
            ),
            const Gap(_bodyTopSpacing),
          ],
        ],
        RichText(
          text: TextSpan(
            children: [
              if (price != null) ...[
                TextSpan(
                  text: price.text.data,
                  style: price.text.style,
                ),
              ],
              if (pricing?.hasAsterisk ?? false) ...[
                WidgetSpan(
                  child: Transform.translate(
                    offset: const Offset(5, -15),
                    child: Text(
                      '*  ',
                      style: titleStyle?.copyWith(fontSize: 18),
                    ),
                  ),
                ),
              ],
              if (period != null) ...[
                TextSpan(
                  text: period.text.data,
                  style: period.text.style,
                ),
              ],
            ],
          ),
        ),
      ],
    );
  }
}
